# Repository path:    $HeadURL: https://90.145.156.222/DMS-Houtribdijk/code/python-ftp-svn/source/MonitorChecks.py $
# Last committed:     $Revision: 279039 $; $Author: jansink $; $Date: 2019-08-19 11:31:01 +0200 (Mon, 19 Aug 2019) $
#
from misc.Utils import Utils
from dmsLogger.DmsLogger import DmsLogger

import pandas as pd
import json
import sys
import getopt

from hkvportal.io.services import dataportal

from checks.checkDMS_difs import checkDMS_difs
from checks.checkDMS_abs import checkDMS_abs

def prtg(type, df):
  
    df_msg = df  

    # add columns for prtg
    if type == 'dif':
        df_msg.loc[:, "channel"] = df_msg["Instrument"] + " difference between " + df_msg["Location1"] + " and " + df_msg["Location2"] + " at " + df_msg["Date"]
    else:
        df_msg.loc[:, "channel"] = df_msg["Instrument"] + " abs "+ df_msg["Location"] + " at " + df_msg["Date"]

    df_msg.loc[:, "value_error"] = df_msg["inRange"]
    df_msg.loc[:, "limitmaxerror"] = 1
    df_msg.loc[:, "limitmode"] = 1
    df_msg.loc[:, "limiterrormsg"] = df_msg['errStr']

    # filter columns to output
    df_msg_out = df_msg[
        ["channel", "value_error", "limitmaxerror", "limitmode", "limiterrormsg"]
    ]
    df_msg_out.columns = [
        "channel",
        "value",
        "limitmaxerror",
        "limitmode",
        "limiterrormsg",
    ]

    return df_msg_out

def all_checks(settings):

    logger = DmsLogger.set_logger(
        settings["log_folder"], settings["logger"], settings["logfile"]
    )
    logger.info("Run MonitorChecks")

    # read settings
    thredds_url = settings["thredds_url"]
    checks_abs = settings["checks_abs"]
    checks_dif = settings["checks_dif"]
    file_monitor_out= settings["file_monitor_out"]
    hkvservices = settings["hkvservices"]
    database = settings["database"]

    dp = dataportal()
    dp.setDataservice(dataservice=hkvservices)

    # run once the following line of code to generate the database
    #dp.createDatabase(database = database)

    df_dif = checkDMS_difs(logger, thredds_url, checks_dif)
    # save data in database
    dp.setEntryDatabase(database=database,
                        key='check_dif',
                        data=df_dif.to_csv(),
                        description='Check dif')
    df_msg_dif = prtg('dif', df_dif)

    df_abs = checkDMS_abs(logger, thredds_url, checks_abs)
    # save data in database
    dp.setEntryDatabase(database=database,
                        key='check_abs',
                        data=df_abs.to_csv(),
                        description='Check abs')
    df_msg_abs = prtg('abs', df_abs)

    df_msg_out = pd.concat([df_msg_dif, df_msg_abs], axis=0, sort=True)

    # save to file
    error_out = {"prtg": {"result": df_msg_out.to_dict(orient="records")}}
    with open(file_monitor_out, "w") as f:
        print(json.dumps(error_out), file=f)

    logger.info("End of MonitorChecks")


def main(argv):
    # input argument checking
    try:
        opts, args = getopt.getopt(argv, "hs:", ["settings="])
    except getopt.GetoptError:
        print("MonitorChecks.py -s <settings_file>")
        sys.exit(2)
    for opt, arg in opts:
        if opt == "-h":
            print("MonitorChecks.py -s <settings_file>")
            sys.exit()
        elif opt in ("-s", "--settings"):
            settings_file = arg

    try:
        settings = Utils.read_monitor_checks_settings(settings_file)
        all_checks(settings)
    except FileNotFoundError:
        exit(1)


if __name__ == "__main__":
    main(sys.argv[1:])
